@@ -7,7 +7,7 @@ fi
 
 CHECK_DEPS xterm
 
-export UUT_EXE="xterm -j -u8 -sl 0 -fc 5 -fg white -bg black -geometry 80x24"
+export UUT_EXE="xterm -kt vt220 -j -u8 -sl 0 -fc 5 -fg white -bg black -geometry 80x24"
 
 # set to non-empty to enable test paths that rely on VT52 support
 export SUPPORTS_VT52=yes
